@@ -13,7 +13,9 @@
 # under the License.
 
 import copy
+import os.path
 import warnings
+import urllib
 
 from keystoneauth1 import discover
 import keystoneauth1.exceptions.catalog
@@ -21,7 +23,6 @@ from keystoneauth1.loading import adapter as ks_load_adap
 from keystoneauth1 import session as ks_session
 import os_service_types
 import requestsexceptions
-from six.moves import urllib
 try:
     import statsd
 except ImportError:
@@ -201,7 +202,7 @@ def from_conf(conf, session=None, service_types=None, **kwargs):
         session=session, config=config_dict, **kwargs)
 
 
-class CloudRegion(object):
+class CloudRegion:
     # TODO(efried): Doc the rest of the kwargs
     """The configuration for a Region of an OpenStack Cloud.
 
@@ -338,7 +339,7 @@ class CloudRegion(object):
         if insecure:
             verify = False
         if verify and cacert:
-            verify = cacert
+            verify = os.path.expanduser(cacert)
         else:
             if cacert:
                 warnings.warn(
@@ -349,8 +350,9 @@ class CloudRegion(object):
 
         cert = self.config.get('cert')
         if cert:
+            cert = os.path.expanduser(cert)
             if self.config.get('key'):
-                cert = (cert, self.config.get('key'))
+                cert = (cert, os.path.expanduser(self.config.get('key')))
         return (verify, cert)
 
     def get_services(self):
@@ -781,7 +783,6 @@ class CloudRegion(object):
                     raise exceptions.ConfigException(
                         "A default microversion for service {service_type} of"
                         " {default_microversion} was requested, but the cloud"
-                        " only supports a maximum of"
                         " only supports a minimum of {min_microversion} and"
                         " a maximum of {max_microversion}. The default"
                         " microversion was set because a microversion"
