@@ -50,6 +50,8 @@ from openstack import utils
 
 _SEEN_FORMAT = '{name}_seen'
 
+LOG = _log.setup_logging(__name__)
+
 
 def _convert_type(value, data_type, list_type=None):
     # This should allow handling list of dicts that have their own
@@ -89,8 +91,18 @@ class _BaseComponent:
     # The class to be used for mappings
     _map_cls = dict
 
+    #: Marks the property as deprecated.
+    deprecated = False
+    #: Deprecation reason message used to warn users when deprecated == True
+    deprecation_reason = None
+
+    #: Control field used to manage the deprecation warning. We want to warn
+    # only once when the attribute is retrieved in the code.
+    already_warned_deprecation = False
+
     def __init__(self, name, type=None, default=None, alias=None, aka=None,
                  alternate_id=False, list_type=None, coerce_to_default=False,
+                 deprecated=False, deprecation_reason=None,
                  **kwargs):
         """A typed descriptor for a component that makes up a Resource
 
@@ -115,6 +127,11 @@ class _BaseComponent:
             If the Component is None or not present, force the given default
             to be used. If a default is not given but a type is given,
             construct an empty version of the type in question.
+        :param deprecated:
+            Indicates if the option is deprecated. If it is, we display a
+            warning message to the user.
+        :param deprecation_reason:
+            Custom deprecation message.
         """
         self.name = name
         self.type = type
@@ -128,6 +145,9 @@ class _BaseComponent:
         self.list_type = list_type
         self.coerce_to_default = coerce_to_default
 
+        self.deprecated = deprecated
+        self.deprecation_reason = deprecation_reason
+
     def __get__(self, instance, owner):
         if instance is None:
             return self
@@ -137,6 +157,7 @@ class _BaseComponent:
         try:
             value = attributes[self.name]
         except KeyError:
+            value = self.default
             if self.alias:
                 # Resource attributes can be aliased to each other. If neither
                 # of them exist, then simply doing a
@@ -156,15 +177,29 @@ class _BaseComponent:
                     setattr(instance, seen_flag, True)
                     value = getattr(instance, self.alias)
                     delattr(instance, seen_flag)
-                    return value
-            return self.default
+            self.warn_if_deprecated_property(value)
+            return value
 
         # self.type() should not be called on None objects.
         if value is None:
             return None
 
+        self.warn_if_deprecated_property(value)
         return _convert_type(value, self.type, self.list_type)
 
+    def warn_if_deprecated_property(self, value):
+        deprecated = object.__getattribute__(self, 'deprecated')
+        deprecate_reason = object.__getattribute__(self, 'deprecation_reason')
+
+        if value and deprecated and not self.already_warned_deprecation:
+            self.already_warned_deprecation = True
+            if not deprecate_reason:
+                LOG.warning("The option [%s] has been deprecated. "
+                            "Please avoid using it.", self.name)
+            else:
+                LOG.warning(deprecate_reason)
+        return value
+
     def __set__(self, instance, value):
         if self.coerce_to_default and value is None:
             value = self.default
@@ -207,7 +242,7 @@ class Computed(_BaseComponent):
     key = "_computed"
 
 
-class _ComponentManager(collections.MutableMapping):
+class _ComponentManager(collections.abc.MutableMapping):
     """Storage of a component type"""
 
     def __init__(self, attributes=None, synchronized=False):
@@ -348,7 +383,7 @@ class QueryParameters:
             # single-argument (like int) and double-argument type functions.
             try:
                 provide_resource_type = (
-                    len(inspect.getargspec(type_).args) > 1)
+                    len(inspect.getfullargspec(type_).args) > 1)
             except TypeError:
                 provide_resource_type = False
 
@@ -562,6 +597,14 @@ class Resource(dict):
             self._computed.attributes == comparand._computed.attributes
         ])
 
+    def warning_if_attribute_deprecated(self, attr, value):
+        if value and self.deprecated:
+            if not self.deprecation_reason:
+                LOG.warning("The option [%s] has been deprecated. "
+                            "Please avoid using it.", attr)
+            else:
+                LOG.warning(self.deprecation_reason)
+
     def __getattribute__(self, name):
         """Return an attribute on this instance
 
@@ -575,7 +618,9 @@ class Resource(dict):
             else:
                 try:
                     return self._body[self._alternate_id()]
-                except KeyError:
+                except KeyError as e:
+                    LOG.debug("Attribute [%s] not found in [%s]: %s.",
+                              self._alternate_id(), self._body, e)
                     return None
         else:
             try:
@@ -1684,9 +1729,18 @@ class Resource(dict):
             allow_unknown_params=allow_unknown_params)
         query_params = cls._query_mapping._transpose(params, cls)
         uri = base_path % params
+        uri_params = {}
 
         limit = query_params.get('limit')
 
+        for k, v in params.items():
+            # We need to gather URI parts to set them on the resource later
+            if (
+                hasattr(cls, k)
+                and isinstance(getattr(cls, k), URI)
+            ):
+                uri_params[k] = v
+
         # Track the total number of resources yielded so we can paginate
         # swift objects
         total_yielded = 0
@@ -1701,7 +1755,7 @@ class Resource(dict):
             data = response.json()
 
             # Discard any existing pagination keys
-            query_params.pop('marker', None)
+            last_marker = query_params.pop('marker', None)
             query_params.pop('limit', None)
 
             if cls.resources_key:
@@ -1720,6 +1774,8 @@ class Resource(dict):
                 # Resource initializer. "self" is already the first
                 # argument and is practically a reserved word.
                 raw_resource.pop("self", None)
+                # We want that URI props are available on the resource
+                raw_resource.update(uri_params)
 
                 value = cls.existing(
                     microversion=microversion,
@@ -1732,6 +1788,17 @@ class Resource(dict):
             if resources and paginated:
                 uri, next_params = cls._get_next_link(
                     uri, response, data, marker, limit, total_yielded)
+                try:
+                    if next_params['marker'] == last_marker:
+                        # If next page marker is same as what we were just
+                        # asked something went terribly wrong. Some ancient
+                        # services had bugs.
+                        raise exceptions.SDKException(
+                            'Endless pagination loop detected, aborting'
+                        )
+                except KeyError:
+                    # do nothing, exception handling is cheaper then "if"
+                    pass
                 query_params.update(next_params)
             else:
                 return
@@ -1818,22 +1885,26 @@ class Resource(dict):
         return the_result
 
     @classmethod
-    def find(cls, session, name_or_id, ignore_missing=True, **params):
+    def find(
+        cls, session, name_or_id, ignore_missing=True,
+        list_base_path=None, **params
+    ):
         """Find a resource by its name or id.
 
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
         :param name_or_id: This resource's identifier, if needed by
-                           the request. The default is ``None``.
+            the request. The default is ``None``.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the resource does not exist.  When set to ``True``, None will be
+            returned when attempting to find a nonexistent resource.
+        :param str list_base_path: base_path to be used when need listing
+            resources.
         :param dict params: Any additional parameters to be passed into
-                            underlying methods, such as to
-                            :meth:`~openstack.resource.Resource.existing`
-                            in order to pass on URI parameters.
+            underlying methods, such as to
+            :meth:`~openstack.resource.Resource.existing` in order to pass on
+            URI parameters.
 
         :return: The :class:`Resource` object matching the given name or id
                  or None if nothing matches.
@@ -1850,9 +1921,14 @@ class Resource(dict):
                 connection=session._get_connection(),
                 **params)
             return match.fetch(session, **params)
-        except exceptions.NotFoundException:
+        except (exceptions.NotFoundException, exceptions.BadRequestException):
+            # NOTE(gtema): There are few places around openstack that return
+            # 400 if we try to GET resource and it doesn't exist.
             pass
 
+        if list_base_path:
+            params['base_path'] = list_base_path
+
         if ('name' in cls._query_mapping._mapping.keys()
                 and 'name' not in params):
             params['name'] = name_or_id
@@ -2014,7 +2090,6 @@ def wait_for_status(session, resource, status, failures, interval=None,
     :raises: :class:`~AttributeError` if the resource does not have a status
              attribute
     """
-    log = _log.setup_logging(__name__)
 
     current_status = getattr(resource, attribute)
     if _normalize_status(current_status) == status.lower():
@@ -2048,7 +2123,7 @@ def wait_for_status(session, resource, status, failures, interval=None,
                 "{name} transitioned to failure state {status}".format(
                     name=name, status=new_status))
 
-        log.debug('Still waiting for resource %s to reach state %s, '
+        LOG.debug('Still waiting for resource %s to reach state %s, '
                   'current state is %s', name, status, new_status)
 
 
